
# This file was generated by 'versioneer.py' (0.28) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2026-06-30T17:28:47-0500",
 "dirty": true,
 "error": null,
 "full-revisionid": "aacf44424ddb90580c338de35f04be65c07928ea",
 "version": "0.66.0+0.gaacf44424.dirty"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)



import weakref
from collections.abc import Callable as _Callable
from collections.abc import Mapping, Sequence
from typing import Any, Final, Hashable, Iterable, NamedTuple, Self, TypeAlias

from _typeshed import Incomplete
from typing_extensions import Generic, TypeVar, override

from numba.core import dispatcher
from numba.core.typing.context import Context
from numba.core.typing.templates import FunctionTemplate, Signature

from .abstract import Callable, DTypeSpec, Dummy, Literal, Type
from .common import Opaque

_T_co = TypeVar("_T_co", covariant=True)
_ThisT = TypeVar("_ThisT", bound=Type)
_ThisT_co = TypeVar("_ThisT_co", bound=Type, default=Type, covariant=True)
_TemplateTypeT = TypeVar("_TemplateTypeT", bound=_TemplateType)
_DispatcherT_co = TypeVar(
    "_DispatcherT_co",
    bound=dispatcher.Dispatcher,
    default=dispatcher.Dispatcher,
    covariant=True,
)
_TupleT_co = TypeVar(
    "_TupleT_co",
    bound=tuple[Any, ...],
    default=tuple[Any, ...],
    covariant=True,
)

_TemplateType: TypeAlias = type[FunctionTemplate]
_AnyCallable: TypeAlias = _Callable[..., Any]
_BoundFunctionKey: TypeAlias = tuple[Hashable, _ThisT, _Callable | None]
_GetPointerFn: TypeAlias = _Callable[[Any], int]

###

def argsnkwargs_to_str(args: Iterable[object], kwargs: Mapping[str, object]) -> str: ...

class BaseFunction(Callable):
    templates: Final[tuple[_TemplateType, ...]]
    typing_key: Final[Hashable]

    @override  # this free `_TemplateT` is needed because `list` is invariant
    def __init__(
        self,
        template: tuple[_TemplateTypeT, ...] | list[_TemplateTypeT] | _TemplateTypeT,
    ) -> None: ...

    #
    @property
    @override
    def key(self) -> tuple[Incomplete, tuple[_TemplateType, ...]]: ...

    #
    @override
    def get_impl_key(self, sig: Signature) -> Hashable: ...
    @override
    def get_call_type(
        self,
        context: Context,
        args: tuple[Type, ...],
        kws: Mapping[str, Type],
    ) -> Signature | None: ...
    @override
    def get_call_signatures(self) -> tuple[Sequence[Signature], bool]: ...

class Function(BaseFunction, Opaque): ...

class BoundFunction(Callable, Opaque, Generic[_ThisT_co]):
    template: Final[_TemplateType]
    typing_key: Final[Hashable]
    this: _ThisT_co

    @override
    def __init__(self, template: _TemplateType, this: _ThisT_co) -> None: ...

    #
    @override
    def unify(
        self,
        typingctx: Context,
        other: _ThisT,
    ) -> BoundFunction[_ThisT_co | _ThisT] | None: ...

    #
    def copy(self, this: _ThisT) -> BoundFunction[_ThisT]: ...

    #
    @property
    @override
    def key(self) -> _BoundFunctionKey[_ThisT_co]: ...
    @override
    def get_impl_key(self, sig: Signature) -> Hashable: ...
    @override
    def get_call_type(
        self,
        context: Context,
        args: tuple[Type, ...],
        kws: Mapping[str, Type],
    ) -> Signature | None: ...
    @override
    def get_call_signatures(self) -> tuple[Sequence[Signature], bool]: ...

class MakeFunctionLiteral(Literal, Opaque): ...

class WeakType(Type, Generic[_T_co]):
    @property
    @override
    def key(self) -> weakref.ref[_T_co]: ...

    #
    @override
    def __eq__(self, other: Self) -> bool: ...  # type: ignore[override]
    @override
    def __hash__(self) -> int: ...

class Dispatcher(WeakType[_DispatcherT_co], Callable, Dummy, Generic[_DispatcherT_co]):
    @override
    def __init__(self, dispatcher: _DispatcherT_co) -> None: ...

    #
    @property
    def dispatcher(self) -> _DispatcherT_co: ...

    #
    @override
    def get_call_type(
        self,
        context: Context,
        args: tuple[Type, ...],
        kws: Mapping[str, Type],
    ) -> Signature | None: ...
    @override
    def get_call_signatures(self) -> tuple[Sequence[Signature], bool]: ...
    @override
    def get_impl_key(self, sig: Signature) -> _AnyCallable: ...
    def get_overload(self, sig: Signature) -> _AnyCallable: ...

class ObjModeDispatcher(Dispatcher[_DispatcherT_co], Generic[_DispatcherT_co]): ...

class ExternalFunctionPointer(BaseFunction):
    sig: Final[Signature]
    requires_gil: Final[bool]
    get_pointer: Final[_GetPointerFn]
    cconv: Final[str | None]

    @override
    def __init__(
        self,
        sig: Signature,
        get_pointer: _GetPointerFn,
        cconv: str | None = None,
    ) -> None: ...

    #
    @property
    @override
    def key(self) -> tuple[Signature, str | None, _GetPointerFn]: ...  # type: ignore[override]

class ExternalFunction(Function):
    symbol: Final[str]
    sig: Final[Signature]

    @override
    def __init__(self, symbol: str, sig: Signature) -> None: ...

    #
    @property
    @override
    def key(self) -> tuple[str, Signature]: ...  # type: ignore[override]

class NamedTupleClass(Callable, Opaque, Generic[_TupleT_co]):
    instance_class: type[_TupleT_co]

    @override
    def __init__(self, instance_class: type[_TupleT_co]) -> None: ...

    #
    @property
    @override
    def key(self) -> type[_TupleT_co]: ...

    #
    @override
    def get_call_type(
        self,
        context: Context,
        args: tuple[Type, ...],
        kws: Mapping[str, Type],
    ) -> None: ...
    @override
    def get_call_signatures(self) -> tuple[tuple[()], bool]: ...
    @override
    def get_impl_key(self, sig: Signature) -> type[Self]: ...

class NumberClass(Callable, DTypeSpec, Opaque, Generic[_T_co]):
    instance_type: _T_co

    @override
    def __init__(self, instance_type: _T_co) -> None: ...

    #
    @property
    @override
    def key(self) -> _T_co: ...
    @property
    @override
    def dtype(self) -> _T_co: ...

    #
    @override
    def get_call_type(
        self,
        context: Context,
        args: tuple[Type, ...],
        kws: Mapping[str, Type],
    ) -> None: ...
    @override
    def get_call_signatures(self) -> tuple[tuple[()], bool]: ...
    @override
    def get_impl_key(self, sig: Signature) -> type[Self]: ...

class _RecursiveCallOverloads(NamedTuple):
    qualname: str
    uid: int

class RecursiveCall(Opaque, Generic[_DispatcherT_co]):
    dispatcher_type: Dispatcher[_DispatcherT_co]

    @override
    def __init__(self, dispatcher_type: Dispatcher[_DispatcherT_co]) -> None: ...
    @property
    @override
    def key(self) -> Dispatcher[_DispatcherT_co]: ...

    #
    def add_overloads(
        self,
        args: tuple[Type, ...],
        qualname: str,
        uid: int,
    ) -> None: ...
    def get_overloads(self, args: tuple[Type, ...]) -> _RecursiveCallOverloads: ...



import abc
from collections.abc import Collection, Sequence
from typing import (
    Any,
    ClassVar,
    Final,
    Iterable,
    Never,
    TypeAlias,
    runtime_checkable,
    type_check_only,
)

from typing_extensions import Protocol, Self, TypeVar, override

from numba.core.compiler import CompileResult
from numba.core.dispatcher import Dispatcher
from numba.core.typing.context import Context
from numba.core.typing.templates import Signature

from .abstract import Type

__all__ = [
    "FunctionType",
    "UndefinedFunctionType",
    "FunctionPrototype",
    "WrapperAddressProtocol",
    "CompileResultWAP",
]

_T = TypeVar("_T")
_T_co = TypeVar("_T_co", covariant=True)

@type_check_only
class _CanUnliteral(Protocol[_T_co]):
    def __unliteral__(self, /) -> _T_co: ...

@type_check_only
class _HasLiteralType(Protocol[_T_co]):
    @property
    def literal_type(self, /) -> _T_co: ...

# represents the return type of `types.unliteral` for literals
_LiteralLike: TypeAlias = _CanUnliteral[_T] | _HasLiteralType[_T]

###

class FunctionType(Type):
    cconv: ClassVar[None] = None

    nargs: Final[int]
    signature: Final[Signature]
    ftype: Final[FunctionPrototype]
    _key: str

    @override
    def __init__(self, signature: Signature | _LiteralLike[Signature]) -> None: ...

    #
    @property
    @override
    def key(self) -> str: ...
    @property  # type: ignore[misc]
    @override
    def name(self) -> str: ...

    #
    def is_precise(self) -> bool: ...
    def get_precise(self) -> FunctionType: ...
    def dump(self, tab: str = "") -> None: ...
    def get_call_type(
        self,
        context: Context,
        args: Sequence[Type],
        kws: dict[Never, Never] | None,  # empty dict or None
    ) -> Signature: ...
    def check_signature(self, other_sig: Signature) -> bool: ...
    def unify(self, context: Context, other: Type) -> Self | None: ...

class UndefinedFunctionType(FunctionType):
    dispatchers: Final[Collection[Dispatcher]]

    @override
    def __init__(
        self,
        nargs: int,
        dispatchers: Collection[Dispatcher],
    ) -> None: ...

class FunctionPrototype(Type):
    cconv: ClassVar[None] = None
    rtype: Type
    atypes: tuple[Type, ...]

    @override
    def __init__(self, rtype: Type, atypes: Iterable[Type]) -> None: ...

    #
    @property
    @override
    def key(self) -> str: ...

@runtime_checkable
class WrapperAddressProtocol(Protocol):
    @abc.abstractmethod
    def __wrapper_address__(self) -> int: ...
    @abc.abstractmethod
    def signature(self) -> Signature: ...

class CompileResultWAP(WrapperAddressProtocol):
    def __init__(self, cres: CompileResult) -> None: ...
    def dump(self, tab: str = "") -> None: ...

    #
    @override
    def __wrapper_address__(self) -> int: ...
    @override
    def signature(self) -> Signature: ...

    #
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
